@@ -17,13 +17,15 @@
 
 from __future__ import division, print_function, absolute_import
 
+import enum
 import math
 import datetime as dt
 import operator
 from decimal import Context, Decimal
-from inspect import isclass
+from inspect import isclass, isfunction
 from numbers import Rational
 from fractions import Fraction
+from functools import reduce
 
 from hypothesis.errors import InvalidArgument, ResolutionFailed
 from hypothesis.control import assume
@@ -401,23 +403,30 @@ def tuples(*args):
 
 @defines_strategy
 def sampled_from(elements):
-    """Returns a strategy which generates any value present in the iterable
-    elements.
+    """Returns a strategy which generates any value present in ``elements``.
 
-    Note that as with just, values will not be copied and thus you
-    should be careful of using mutable data.
+    Note that as with :func:`~hypotheses.strategies.just`, values will not be
+    copied and thus you should be careful of using mutable data.
+
+    ``sampled_from`` supports ordered collections, as well as
+    :class:`~python:enum.Enum` objects.  :class:`~python:enum.Flag` objects
+    may also generate any combination of their members.
 
     """
-
-    from hypothesis.searchstrategy.misc import SampledFromStrategy, \
-        JustStrategy
+    from hypothesis.searchstrategy.misc import SampledFromStrategy
     from hypothesis.internal.conjecture.utils import check_sample
-    elements = check_sample(elements)
-    if not elements:
+    values = check_sample(elements)
+    if not values:
         return nothing()
-    if len(elements) == 1:
-        return JustStrategy(elements[0])
-    return SampledFromStrategy(elements)
+    if len(values) == 1:
+        return just(values[0])
+    if hasattr(enum, 'Flag') and isclass(elements) and \
+            issubclass(elements, enum.Flag):
+        # Combinations of enum.Flag members are also members.  We generate
+        # these dynamically, because static allocation takes O(2^n) memory.
+        return sets(sampled_from(values), min_size=1).map(
+            lambda s: reduce(operator.or_, s))
+    return SampledFromStrategy(values)
 
 
 @cacheable
@@ -927,11 +936,18 @@ def from_type(thing):
     """
     from hypothesis.searchstrategy import types
     if not isinstance(thing, type):
-        # Under Python 3.6, Unions are not instances of `type` - but we still
-        # want to resolve them!  This __origin__ check only passes if thing is
-        # a Union with parameters; if it doesn't we can't resolve it anyway.
         try:
+            # At runtime, `typing.NewType` returns an identity function rather
+            # than an actual type, but we can check that for a possible match
+            # and then read the magic attribute to unwrap it.
             import typing
+            if all([
+                hasattr(thing, '__supertype__'), hasattr(typing, 'NewType'),
+                isfunction(thing), getattr(thing, '__module__', 0) == 'typing'
+            ]):
+                return from_type(thing.__supertype__)
+            # Under Python 3.6, Unions are not instances of `type` - but we
+            # still want to resolve them!
             if getattr(thing, '__origin__', None) is typing.Union:
                 args = sorted(thing.__args__, key=types.type_sorting_key)
                 return one_of([from_type(t) for t in args])
@@ -987,6 +1003,9 @@ def from_type(thing):
             and hasattr(thing, '_field_types'):
         kwargs = {k: from_type(thing._field_types[k]) for k in thing._fields}
         return builds(thing, **kwargs)
+    if issubclass(thing, enum.Enum):
+        assert len(thing), repr(thing) + ' has no members to sample'
+        return sampled_from(thing)
     # If the constructor has an annotation for every required argument,
     # we can (and do) use builds() without supplying additional arguments.
     required = required_args(thing)
@@ -1681,7 +1700,7 @@ def try_convert(typ, value, name):
                 name, value, type(value).__name__, typ.__name__
             )
         )
-    except ValueError:
+    except (OverflowError, ValueError):
         raise InvalidArgument(
             'Cannot convert %s=%r to type %s' % (
                 name, value, typ.__name__
